@@ -1,21 +1,25 @@
 use std::mem::swap;
 use std::ops::{Index, IndexMut};
 
-use imbl::{HashSet, Vector};
+use imbl::Vector;
 
 // todo: the current implementation of this immutable arena is very naively implemented – we're simply copying how it would be implemented with a regular Vec, and swapping it out for an immutable Vector. there are likely much better implementations possible with a custom clever immutable data structure. don't do anything before benchmarking though
 
 /// A persistent Arena – a collection of items with stable IDs
 #[derive(Clone, Debug)]
+#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 pub struct Arena<T: Clone> {
     entries: Vector<Entry<T>>,
     first_free: Option<EntryId>,
 }
 
 #[derive(Copy, Clone, Eq, PartialEq, Ord, PartialOrd, Hash, Debug)]
+#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
+/// The ID of an entry in an Arena
 pub struct EntryId(pub(crate) usize);
 
 #[derive(Clone, Debug)]
+#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 enum Entry<T> {
     Some(T),
     Free { next_free: Option<EntryId> },
@@ -39,7 +43,10 @@ impl<T: Clone> Arena<T> {
     /// - Not be circular
     ///
     /// Panics if the state is invalid
-    pub fn validate(&self) {
+    #[cfg(test)]
+    pub(crate) fn validate(&self) {
+        use std::collections::HashSet;
+
         let expected_free_indices: HashSet<_> = self
             .entries
             .iter()
@@ -54,9 +61,9 @@ impl<T: Clone> Arena<T> {
 
         let mut free_cursor = self.first_free;
         while let Some(index) = free_cursor {
-            let old_value = found_free_indices.insert(index);
+            let had_value = !found_free_indices.insert(index);
 
-            if old_value.is_some() {
+            if had_value {
                 panic!("Circular Free list: index {:?} encountered twice", index)
             }
 
