@@ -1,10 +1,10 @@
 use crate::tree::NodeId;
 
-#[derive(Debug)]
+#[derive(Debug, Copy, Clone)]
 /// An error that occurs when attempting to remove the root node from a tree
 pub struct CannotRemoveRoot;
 
-#[derive(Debug)]
+#[derive(Debug, Copy, Clone)]
 /// An error that occurs when attempting to move a node to an invalid location
 pub enum MoveError<Id> {
     /// The specified location referenced a node ID that doesn't exist
@@ -17,13 +17,16 @@ pub enum MoveError<Id> {
     CannotMoveUnderSelf,
 }
 
-#[derive(Debug)]
-pub(crate) enum InvalidLocation {
-    NoSuchNode(NodeId),
+#[derive(Debug, Copy, Clone)]
+/// An error that occurs when attempting to insert a node in an invalid location
+pub enum InvalidLocation<Id> {
+    /// The specified location referenced a node ID that doesn't exist
+    NoSuchNode(Id),
+    /// Attempted to place the node as a sibling of the root node; this is not allowed
     RootCannotHaveSiblings,
 }
 
-#[derive(Debug)]
+#[derive(Debug, Copy, Clone)]
 pub(crate) enum RemoveByIdError {
     NoSuchNode(NodeId),
     CannotRemoveRoot,
