@@ -1,5 +1,3 @@
-use imbl::HashSet;
-
 use crate::arena::{Arena, EntryId};
 pub use children::*;
 pub use node_mut::*;
@@ -23,12 +21,14 @@ mod node_ref;
 ///
 /// A node's children is logically a list – so the order of the children is specified, and you can change it if you want.
 #[derive(Clone, Debug)]
+#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 pub struct Tree<T: Clone> {
     root: NodeId,
     pub(crate) arena: Arena<Node<T>>,
 }
 
 #[derive(Clone, Debug)]
+#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 pub(crate) struct Node<T: Clone> {
     data: T,
     parent: Option<NodeId>,
@@ -49,7 +49,9 @@ impl<T: Clone> Node<T> {
     }
 }
 
+/// The ID of a node in a Tree
 #[derive(Copy, Clone, Eq, PartialEq, Ord, PartialOrd, Hash, Debug)]
+#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 pub struct NodeId(pub(crate) EntryId);
 
 impl<T: Clone> Tree<T> {
@@ -61,11 +63,52 @@ impl<T: Clone> Tree<T> {
         Self { root, arena }
     }
 
-    fn insert(&mut self, value: T) -> NodeId {
+    fn create_node(&mut self, value: T) -> NodeId {
         let id = self.arena.insert(Node::new(value));
         NodeId(id)
     }
 
+    fn validate_location(&self, location: Location<NodeId>) -> Result<(), InvalidLocation<NodeId>> {
+        match location {
+            Location::AfterSibling(id) => {
+                if id == self.root {
+                    return Err(InvalidLocation::RootCannotHaveSiblings);
+                }
+
+                self.arena
+                    .get(id.0)
+                    .ok_or(InvalidLocation::NoSuchNode(id))?;
+            }
+            Location::FirstChildOf(id) => {
+                self.arena
+                    .get(id.0)
+                    .ok_or(InvalidLocation::NoSuchNode(id))?;
+            }
+        }
+
+        Ok(())
+    }
+
+    /// Insert a new node at the specified location
+    ///
+    /// Error if the location is invalid:
+    /// - The root node cannot have siblings
+    /// - The location must reference only existing nodes
+    pub fn insert(
+        &mut self,
+        value: T,
+        location: Location<NodeId>,
+    ) -> Result<NodeId, InvalidLocation<NodeId>> {
+        // annoying: gets validated twice. todo.
+        self.validate_location(location)?;
+
+        let node = self.create_node(value);
+        self.attach(node, location)
+            .expect("unreachable; location validated");
+
+        Ok(node)
+    }
+
     /// Attach the specified node at the specified location.
     ///
     /// This will update the relevant references of siblings, the parent, and the specified node.
@@ -73,7 +116,7 @@ impl<T: Clone> Tree<T> {
         &mut self,
         node_id: NodeId,
         location: Location<NodeId>,
-    ) -> Result<(), InvalidLocation> {
+    ) -> Result<(), InvalidLocation<NodeId>> {
         match location {
             Location::AfterSibling(previous_sibling_id) => {
                 // todo: repeatedly accessing the same entries, could cache with Focus
@@ -165,7 +208,10 @@ impl<T: Clone> Tree<T> {
     /// - The root has no siblings
     ///
     /// Panics if the state is invalid
-    pub fn validate(&self) {
+    #[cfg(test)]
+    pub(crate) fn validate(&self) {
+        use std::collections::HashSet;
+
         self.arena.validate();
 
         fn collect_ids<T: Clone>(
@@ -173,9 +219,9 @@ impl<T: Clone> Tree<T> {
             mut found_ids: &mut HashSet<NodeId>,
             id: NodeId,
         ) {
-            let previous_value = found_ids.insert(id);
+            let had_id = !found_ids.insert(id);
 
-            if previous_value.is_some() {
+            if had_id {
                 panic!("Circular graph: second occurrence of {:?}", id);
             }
 
@@ -268,6 +314,31 @@ mod tests {
         tree.validate();
     }
 
+    #[test]
+    fn insert() {
+        let mut tree = Tree::new_with_root(42);
+
+        let location = Location::FirstChildOf(tree.root().id());
+        assert!(tree.insert(1, location).is_ok());
+
+        let invalid_location = Location::AfterSibling(tree.root().id());
+        assert!(matches!(
+            tree.insert(2, invalid_location),
+            Err(InvalidLocation::RootCannotHaveSiblings)
+        ));
+
+        let invalid_location = Location::FirstChildOf(NodeId(EntryId(404)));
+        assert!(matches!(
+            tree.insert(3, invalid_location),
+            Err(InvalidLocation::NoSuchNode(_))
+        ));
+
+        let children: Vec<_> = tree.root().children().map(|node| *node.data()).collect();
+        assert_eq!(children, &[1]);
+
+        tree.validate();
+    }
+
     #[test]
     fn data() {
         let mut tree = Tree::new_with_root(42);
@@ -368,7 +439,7 @@ mod tests {
 
         let mut node = root.push_front_child(1);
         let node_id = node.id();
-        node.push_front_child(2).push_next_sibling(3);
+        node.push_front_child(2).push_next_sibling(3).unwrap();
 
         let mut removed = HashSet::new();
         tree.get_mut(node_id)
@@ -434,7 +505,7 @@ mod tests {
     fn persistence() {
         let mut tree = Tree::new_with_root(42);
         let old = tree.clone();
-        let index = tree.insert(5);
+        let index = tree.create_node(5);
 
         assert!(tree.get(index).is_some());
         assert!(old.get(index).is_none());
