@@ -2,7 +2,7 @@ use crate::error::{CannotRemoveRoot, MoveError, RemoveByIdError};
 use crate::tree::{Children, NodeId, Tree};
 use crate::tree_location::Location;
 
-/// A mutable reference to a node
+/// A mutable reference to a node in a Tree
 ///
 /// The library guarantees that this references a valid node in the associated tree
 #[derive(Debug)]
@@ -36,6 +36,10 @@ impl<'a, T: Clone> NodeMut<'a, T> {
     }
 }
 
+/// Error: the root node cannot have siblings
+#[derive(Copy, Clone, Debug)]
+pub struct RootCannotHaveSiblings();
+
 impl<'a, T: Clone> NodeMut<'a, T> {
     /// Get a mutable reference to the node's data
     pub fn data_mut(&mut self) -> &mut T {
@@ -50,7 +54,7 @@ impl<'a, T: Clone> NodeMut<'a, T> {
     pub fn push_front_child(&mut self, value: T) -> NodeMut<T> {
         let tree = &mut self.tree;
 
-        let new_id = tree.insert(value);
+        let new_id = tree.create_node(value);
         tree.attach(new_id, Location::FirstChildOf(self.index))
             .expect("Pushing child of existing node cannot fail");
 
@@ -60,17 +64,17 @@ impl<'a, T: Clone> NodeMut<'a, T> {
     /// Push a new child as the next sibling of this node
     ///
     /// This is allowed for all nodes except for the root node. When attempting to add a sibling to the root node, this will return None, and nothing will be added to the tree
-    pub fn push_next_sibling(&mut self, value: T) -> Option<NodeMut<T>> {
+    pub fn push_next_sibling(&mut self, value: T) -> Result<NodeMut<T>, RootCannotHaveSiblings> {
         if self.index == self.tree.root {
-            return None;
+            return Err(RootCannotHaveSiblings());
         }
 
         let tree = &mut self.tree;
 
-        let new_id = tree.insert(value);
+        let new_id = tree.create_node(value);
 
         match tree.attach(new_id, Location::AfterSibling(self.index)) {
-            Ok(_) => Some(NodeMut::new(self.tree, new_id)),
+            Ok(_) => Ok(NodeMut::new(self.tree, new_id)),
             Err(_) => unreachable!(),
         }
     }
