@@ -1,14 +1,16 @@
 use crate::tree::{Children, NodeId, Tree};
 
-/// A reference to a node
+/// A reference to a node in a Tree
 ///
 /// The library guarantees that this references a valid node in the associated tree
-#[derive(Clone, Copy, Debug)]
+#[derive(Clone, Debug)]
 pub struct NodeRef<'a, T: Clone> {
     tree: &'a Tree<T>,
     index: NodeId,
 }
 
+impl<'a, T: Clone> Copy for NodeRef<'a, T> {}
+
 impl<'a, T: Clone> NodeRef<'a, T> {
     pub(crate) fn new(tree: &'a Tree<T>, index: NodeId) -> Self {
         Self { tree, index }
